/****************************************************************************
*  Copyright (C) 2012 by Michael Fischer
*
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
****************************************************************************/
#define __ALTERA_DE0_NANO_C__

#if defined(ALTERA_DE0_NANO)

/*=========================================================================*/
/*  INCLUDE                                                                */
/*=========================================================================*/
#include <stdio.h>
#include <string.h>
#include <compiler.h>
#include <dev/board.h>

#include "system.h"
#include "sys/alt_flash.h"
#include "sys/alt_flash_dev.h"
#include "os/alt_syscall.h"
#include "arch/nios2/nios_io.h"
#include "altera_avalon_uart.h"
#include "altera_avalon_uart_regs.h"

extern unsigned int alt_busy_sleep (unsigned int us);

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Prototypes                                                     */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/
static unsigned char TestBuffer[64];

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/

/***************************************************************************/
/*  Early hardware initialization.                                         */
/***************************************************************************/
void NutBoardInit (void)
{
#if defined(NIC_RESET_PORT) && defined(NIC_RESET_PORT_BIT)
   /* Set NIC reset */
   outw(NIC_RESET_PORT, ~NIC_RESET_PORT_BIT);    
   alt_busy_sleep(100);

   /* Remove NIC reset */
   outw(NIC_RESET_PORT, NIC_RESET_PORT_BIT);    
   alt_busy_sleep(100);
#endif

   /*
    * Disable interrupts which was still enabled
    * from the Niso II system at startup.
    */
   alt_ic_irq_disable(0, UART_0_IRQ);
   IOWR_ALTERA_AVALON_UART_CONTROL(UART_0_BASE, 0);
} /* NutBoardInit */

/***************************************************************************/
/*  DE0NanoReadData                                                        */
/***************************************************************************/
int DE0NanoReadData (unsigned int addr, void *buff, size_t siz)
{
   int           error = -1;
   alt_flash_fd* fd;
   
   fd = alt_flash_open_dev(EPCS_FLASH_CONTROLLER_NAME);
   if (fd != NULL)
   {
      /*
       * Add offset from last sector of the EPCS device 
       */
      addr = addr + 0x7F0000;
      
      error = alt_read_flash(fd, addr, buff, siz); 
      
      alt_flash_close_dev(fd);
   }

   return(error);
} /* DE0NanoReadData */

/***************************************************************************/
/*  DE0NanoWriteData                                                       */
/***************************************************************************/
int DE0NanoWriteData (unsigned int addr, CONST void *buff, size_t siz)
{
   int           error = -1;
   alt_flash_fd* fd;
   
   fd = alt_flash_open_dev(EPCS_FLASH_CONTROLLER_NAME);
   if (fd != NULL)
   {
      /*
       * Add offset from last sector of the EPCS device 
       */
      addr = addr + 0x7F0000;
      
      if (siz <= sizeof(TestBuffer))
      {
         alt_read_flash(fd, addr, TestBuffer, siz); 
         if (memcmp(buff, TestBuffer, siz) == 0)
         {
            alt_flash_close_dev(fd);
            return(0);
         }
      }

      error = alt_write_flash(fd, addr, buff, siz); 
      
      alt_flash_close_dev(fd);
   }

   return(error);
} /* DE0NanoWriteData */

#endif /* if defined(ALTERA_DE0_NANO) */
/*** EOF ***/
