/****************************************************************************
*  Copyright (C) 2012 by Michael Fischer
*
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
****************************************************************************/

#include <cfg/os.h>
#include <sys/atom.h>
#include <dev/irqreg.h>
#include <arch/timer.h>

/* NiosII */
#include <system.h>
#include <sys/alt_alarm.h>

/*
 * Some local variables for the Nios II alarm functionality.
 */
static alt_alarm alarm;
static void (*TimeTick) (void *arg);

/*!
 * \brief NiosII timer callback.
 *
 * This function is NiosII frame for the real TimeTick.
 *
 * \param context Parameter of NutTimerIntr.
 */
static alt_u32 TimerCallback (void* context)
{
   TimeTick(context);
   return(1);
}

/*!
 * \brief Initialize system timer hardware.
 *
 * This function is automatically called by Nut/OS during system 
 * initialization.
 *
 * \param handler System timer interrupt handler.
 */
void NutRegisterTimer(void (*handler) (void *))
{
   TimeTick = handler;
   
   alt_alarm_start(&alarm, 1, &TimerCallback, NULL);
}

/*!
 * \brief Return the CPU clock in Hertz.
 *
 * \return CPU clock frequency in Hertz.
 */
uint32_t NutArchClockGet(int idx)
{
   return(ALT_CPU_FREQ);
}

/*!
 * \brief Return the number of system ticks per second.
 *
 * \return System tick frequency in Hertz.
 */
uint32_t NutGetTickClock(void)
{
   return(SYS_CLK_TIMER_TICKS_PER_SEC);
}

/*!
 * \brief Calculate system ticks for a given number of milliseconds.
 */
uint32_t NutTimerMillisToTicks(uint32_t ms)
{
   return((ms * SYS_CLK_TIMER_TICKS_PER_SEC) / 1000);
}

/*** EOF ***/


